<?php declare (strict_types=1);

namespace app\command;

use app\admin\common\User;
use app\admin\model\ActionLog;
use app\admin\model\GoodLog;
use app\admin\model\GoodStockInfo;
use app\admin\model\OrderOutChild;
use app\admin\model\OrderTax;use app\admin\model\ProcessOrder;
use app\admin\model\SaleCgdPrice;use think\console\Command;
use think\console\Input;
use think\console\Output;
use think\Exception;
use think\facade\Cache;
use think\facade\Config;
use think\facade\Db;

//处理有赞订单数据
class handleYzOrderData extends Command
{
    private $cgd_data = [];//存储采购单的数据，目前是为了做供应商api推送而准备的

    protected function configure()
    {
        // 指令配置
        $this->setName('handle_yz_order_data')->setDescription('处理有赞订单数据');
    }

    //处理有赞订单数据
    protected function execute(Input $input, Output $output)
    {
		$limit = Cache::store('redis')->get("YzTimeLimit");
        if($limit==1){
        	return '' ;
        }
        $c_data = Cache::store('redis')->handler()->rpop(Config::get('app.handle_queue_key'));
        Cache::store('redis')->set("YzTimeLimit",1,60);
        if ($c_data){

            $c_data = json_decode($c_data, true);

            Db::startTrans();
            try {


                $standing_book_data = [];


                $orderCode = makeNo("QR");


                //企业客户
                $ct = Db::name('platform_youzan')
                    ->alias('py')
                    ->field('py.id,py.skuCode,py.spuCode,py.plat_code,p.platform_name,py.sale_price,g.*,py.platform_id')
                    ->leftJoin('platform p', 'p.id=py.platform_id AND p.is_del=0')
                    ->leftJoin('good g', 'g.spuCode=py.spuCode AND g.is_del=0')
                    ->where('py.skuCode', $c_data['skuCode'])
                    ->where('py.exam_status', 6)//exam_status==3已上线
                    ->find();
                if ($ct) {
                    $customer_code = Db::connect('mysql_sys')
                        ->name('customer_info')->alias("a")
                        ->leftJoin("customer_org1 b" ,"a.itemid=b.id and b.is_del=0")
                        ->where(['companyName' => $ct['platform_name'] . '客户', "b.name"=>$ct['platform_name'], 'a.is_del' => 0])
                        ->value('companyNo', '');

                    if (!$customer_code) throw new Exception('在企业客户管理组织中找不到对应的企业');

                } else throw new Exception('未找到商品数据');

                $supplierNo = $ct['companyNo'];

                $goodtype = 1;//1正常商品

                $sendtype = 1;//直接发货
                $platform_id = $ct['platform_id'];

                $platform_order = $c_data['tid'];
                $good_num = $c_data['num'];
                $arrtime = $paytime = $c_data['addtime'];


                $goodinfo = $ct;
                $is_stock = $ct['is_stock'];
                $order_type = $is_stock == 1 ? 1 : 2;//1备库(库存品)2非库存品
                $order_source = 5;//有赞
                $spuCode = $ct['spuCode'];
                $skuCode = $ct['skuCode'];

				$origin = Db::name('good_nake')
				                        ->where([['spuCode', '=', $spuCode], ['min_num', '<=', $good_num], ['is_del', '=', 0]])
				                        ->order('min_num desc')
				                        ->find();
				$origin_price = $goodinfo['is_stock'] == 1?0:($origin['nake_total']??0);
			    $cgd_origin_price = $origin['supplier_origin_price']??0;
                if ($is_stock == 1) {
                    if($ct['usable_stock'] < $good_num){
			               throw new Exception('商品库存数量不足');
			            }
                    $origin_price = 0;

                }else{
                	if ($origin == false) throw new \Exception('未找到相关成本价格');
                }

				$is_activity = 0;
                $sale_price = $c_data['discount_price'];
	            if (isset($c_data['coupon']) && is_discount($c_data['coupon'])) {
		            //如果是泰行销的单子，要取原始价格(即优惠前的价格)
		            $is_activity=2;//txx活动标识
		            $sale_price = $c_data['price'];
	            }


                $act = check_activity((string)$spuCode, (string)$skuCode, (int)$platform_id, (int)$is_stock, (float)$sale_price, (int)$good_num);

                if (!empty($act)) {
                    $is_activity = 1;
                    $sale_price = $act['final_price'];//取活动的相关价格
                }else{

	                if ($sale_price < $ct['sale_price'] && $c_data['is_approved'] == '0') {
		                Db::rollback();
		                $this->callbackYz(['status' => 2, 'id' => $c_data['id'], 'reason' => '价格低于系统售价' . $ct['sale_price']]);
		                Cache::store('redis')->set("YzTimeLimit",0);
		                return false;//停止执行后续代码
	                }

                }



                $remark = $c_data['buyer_messages'];

                $user = Config::get('app.uid_platform_youzan.' . $ct['platform_id']);
                $rm = isset($user['id']) ? $user['id'] : 0;
                $ri = isset($user['nickname']) ? $user['nickname'] : 'youzan';
				if($rm==0)throw new Exception('平台转单未找到对应的平台账户');
				
                $userCommon = User::getIns();
                $tmp = $userCommon->handle('sInfo',['code'=>$ct['supplierNo']]);
                $supplier_temp_info = $tmp['data']??[];

                $names = $userCommon->handle('getCodeAndName',['code'=>[$goodinfo['companyNo']]]);

                $data = [
                    "orderCode" => $orderCode,
                    "good_code" => $spuCode,
                    "skuCode" => $skuCode,
                    "customer_code" => $customer_code,
                    "customerName" => $ct['platform_name'] . '客户',
                    "good_name" => isset($goodinfo['good_name']) && $goodinfo['good_name'] !== '' ? $goodinfo['good_name'] : '',
                    "good_num" => $good_num,
                    "cat_id" => $goodinfo['cat_id'],
                    "apply_id" => $rm,
                    "apply_name" => $ri,
                    "origin_price" => $origin_price,
                    "sale_price" => $sale_price,
                    "post_fee" => 0,
                    "status" => 0,
                    "send_num" => 0,
                    "wsend_num" => $good_num,
                    "send_status" => 1,
                    "good_type" => $goodtype,
                    "send_type" => $sendtype,
                    "supplierNo" => $goodinfo['companyNo'],
                    "supplierName" => $names['data'][$goodinfo['companyNo']]??"",
                    'supNo'=>$goodinfo['supplierNo'],
                    'supName'=>$supplier_temp_info['name'],
                    "is_del" => 0,
                    "zxNo" => "",
                    "platform_order" => $platform_order,
                    "platform_id" => $platform_id,
                    "remark" => $remark,
                    "is_stock" => $is_stock,
                    "is_activity" => $is_activity,
                    'activity_code' => $is_activity==1 ? $act['activity_code'] : ($is_activity==2?"txx":""),
                    "order_type" => $order_type,
                    "order_source" => $order_source,
                    'open_type'=>$goodinfo['open_type'],
                    'good_weight' => $ct['weight'],
                    'gold_price' => $ct['cgd_gold_price'],
                    'cost_price' => $ct['cost_fee'],
                    'diff_weight' => 0,
                    'diff_fee' => 0,
                    "workNo" => '',
                    "addtime" => date("Y-m-d H:i:s"),
                    "updatetime" => date("Y-m-d H:i:s"),
                    'total_price' => bcmul((string)$sale_price, (string)$good_num, 2),
                    'cgderid' => $supplier_temp_info['personid']??'',
                    'cgder' => $supplier_temp_info['person']??'',//采购员(供应商负责人)
                    'good_createrid' => $goodinfo['createrid'],
                    'good_creater' => $goodinfo['creater'],//商品创建人
                     'manager'=>$ri,
                    'managerid'=>$rm,
                ];
                $paytime == "" ? "" : $data['paytime'] = $paytime;
                $datainfo = Db::name('sale')->insertGetId($data);

                if ($datainfo > 0) {
                    if ($is_activity == 1) {
                        $actup = [
                            "activity_stock" => $act['activity_stock'] - $good_num,
                            "updatetime" => date("Y-m-d H:i:s")
                        ];
                        $actupp = Db::name("activity_info")
                            ->where('id', $act['id'])
                            ->save($actup);
                        if ($actupp == false) throw new Exception('活动库存修改失败');

                    }

                    //补充台账数据
                    $standing_book_data = array_merge($standing_book_data, [
                        'orderCode' => $orderCode,
                        'spuCode' => $data['good_code'],
                        'skuCode' => $data['skuCode'],
                        'order_type' => $data['order_type'],
                        'order_source' => $data['order_source'],
                        'supplierNo' => $data['supplierNo'],
                        'companyNo' => $ct['companyNo'],
                        'customer_code' => $data['customer_code'],
                    ]);

                    if ($is_stock == 0) {

                        $cgd = [
                            "supplierNo" => $ct['supplierNo'],
                            "supplier_name" => $supplier_temp_info['name'],
                            "companyNo" => $supplierNo,
                            "companyName" => $names['data'][$supplierNo]??'',
                            "orderCode" => $orderCode,
                            "spuCode" => $spuCode,
                            "skuCode" => $skuCode,
                            "good_name" => $ct['good_name'],
                            "sale_price" => $origin_price,
                            "total_fee" => $origin_price * $good_num,
                            "origin_total" => $origin_price * $good_num,
                            "pakge_fee" => isset($origin['package_fee']) ? $origin['package_fee'] : 0,
                            "cert_fee" => isset($origin['cert_fee']) ? $origin['cert_fee'] : 0,
                            "open_fee" => $ct['open_fee'],
                            "cost_fee" => isset($origin['cost_fee']) ? $origin['cost_fee'] : 0,
                            "mark_fee" => isset($origin['mark_fee']) ? $origin['mark_fee'] : 0,
                            "demo_fee" => $ct['demo_fee'],
                            "nake_fee" => isset($origin['nake_fee']) ? $origin['nake_fee'] : 0,
                            "delivery_fee" => isset($origin['delivery_fee']) ? $origin['delivery_fee'] : 0,
                            "good_num" => $good_num,
                            "good_type" => $goodtype,
                            "open_type"=>$goodinfo['open_type'],
                            "order_type" => $order_type,
                            "order_source" => $order_source,
                            "createrid" =>$supplier_temp_info['personid']??'',
                            "creater" =>$supplier_temp_info['person']??'',
                            'send_way' => 2,
                            'gold_price' => $ct['cgd_gold_price'],
                            'good_createrid' => $goodinfo['createrid'],
                            'good_creater' => $goodinfo['creater'],//商品创建人
                            'weight' => $ct['noble_weight'],//商品创建人
                             'supplier_origin_price'=>$cgd_origin_price,//供应商采购成本
						     'cgd_supplier_code'=>$goodinfo['cgd_supplier_code'],//采购供应商编号
						     'cgd_supplier_name'=>$goodinfo['cgd_supplier_name'],//采购供应商编号
						     'cgd_apply_id'=>$goodinfo['cgd_apply_id'],//竞单人
						     'cgd_apply_name'=>$goodinfo['cgd_apply_name'],//竞单人
                        ];

                        //非库存品
                        $bol = $this->createCgd($cgd, $rm, $ri, $standing_book_data);
                        if ($bol == false) throw new Exception('订单创建失败');

                    } else {

	                        $stockUp =[
		                        "usable_stock"=>$ct['usable_stock']-$good_num,
		                        "updatetime"=>date("Y-m-d H:i:s"),
		                        ];
	                        $update=Db::name("good")->where(["spuCode"=>$ct['spuCode'],"usable_stock"=>$ct['usable_stock']])->update($stockUp);
                            if($update==false){
	                             throw new Exception('商品更新可用库存失败');
	                        }
					SaleCgdPrice::GetPrice($datainfo);
                    }
           $taxInfo = OrderTax::SpuCat( $spuCode);
           if (!empty($taxInfo)) {
              (new \app\admin\model\OrderTax)->saveAll(array_map(function ($item)use($orderCode){
                   $temp=[];
                   $temp['code']=$orderCode;
                   $temp['spuCode']=$item['spuCode'];
                   $temp['tax_code']=$item['cat_code'];
                   $temp['tax_name']=$item['cat_name'];
                   $temp['tax_short_name']=$item['short_name'];
                   $temp['inv_tax']=$item['tax'];
                   $temp['inv_good_name']=$item['inv_good_name'];
                   return $temp;
              },$taxInfo));
            }
                    if ($sendtype == 1) {

//                        $p_c_a = get_address($c_data['delivery_province'] . $c_data['delivery_city'] . $c_data['delivery_district']);

                        $province = Db::name('province')
                            ->where('name', $c_data['delivery_province'])
                            ->value('province_code', '');
                        //地级市可能有重复的，例如'省直辖县'
                        $city = Db::name('city')
                            ->where('name', $c_data['delivery_city'])
                            ->where('province_code', $province)
                            ->value('city_code', '');
                        //县区名称也有可能重复，例如'桥东区'
                        $area = Db::name('area')
                            ->where('name', $c_data['delivery_district'])
                            ->where('city_code', $city)
                            ->value('area_code', '');

                        $addr_code = implode([$province, $city, $area], ',');

                        $temp = [
                            'orderCode' => $orderCode,
                            'contactor' => $c_data['receiver_name'],
                            'mobile' => $c_data['receiver_tel'],
                            'addr' => $c_data['delivery_address'],
                            'addr_code' => $addr_code,
                            'customer_code' => $customer_code,
                            'receipt_quantity' => $c_data['num'],//收货数量,
                            'post_fee' => $c_data['post_fee'],
                            'is_del' => 0,
                            'addtime' => date("Y-m-d H:i:s"),
                            'updatetime' => date("Y-m-d H:i:s"),
                            'arrive_time' => $arrtime,
                        ];
                        $vmp = Db::name('order_addr')->insertGetId($temp);
                        if ($vmp > 0) {
								$num = $c_data['num'];
	                            $outCode = makeNo("DF");
                        	if ($is_stock != 1) {
	                            $order = Db::name("order_num")
	                                ->where(["orderCode" => $orderCode, "status" => 1])
	                                ->find();
	                            if ($order == false) throw new Exception('未找到可以发货得采购单数据');
	                            $tep = [
	                                "cgdNo" => $order['cgdNo'],
	                                "outCode" => $outCode,
	                                "send_num" => $num,
	                                "status" => 1,
	                                "addtime" => date("Y-m-d H:i:s"),
	                                "updatetime" => date("Y-m-d H:i:s")
	                            ];
	                            $sen = Db::name("order_send")->save($tep);
	                            if ($sen == false) throw new Exception('发货地址添加创建失败');

	                            $cgdinfo = Db::name("purchease_order")->where(["cgdNo" => $order['cgdNo']])->find();
	                            if ($cgdinfo == false) throw new Exception('未匹配到采购数据');
							}
                            $out = [
                                "orderCode" => $orderCode,
                                "outCode" => $outCode,
                                "apply_id" => $rm,
                                "apply_name" => $ri,
                                "addrid" => $vmp,
                                "post_name" => "",
                                "post_code" => "",
                                "post_fee" => 0,
                                "sendtime" => date("Y-m-d H:i:s"),
                                "send_num" => $num,
                                "check_num" => 0,
                                "error_num" => 0,
                                "wsm_code" => $is_stock==1? '':$cgdinfo['wsm_code'],
                                "order_type" => $order_type,
                                "status" => $is_stock == 1 ? 1 : 0,
                                "send_status" => $is_stock == 1 ? 1 : 0,
                                "addtime" => date("Y-m-d H:i:s"),
                                "updatetime" => date("Y-m-d H:i:s")
                            ];
                            $ou = Db::name("order_out")->insertGetId($out);
                            if ($ou == false) throw new Exception('发货地址添加创建失败');
                            else {
                            if ($is_stock == 1) OrderOutChild::makeChild($outCode);
                                //修改状态，添加待办
                                ActionLog::logAdd(['id' => $rm, 'nickname' => $ri], [
                                    "order_code" => $outCode,//出库单号
                                    "status" => 0,//这里的status是之前的值
                                    "action_remark" => '',//备注
                                    "action_type" => "create"//新建create，编辑edit，更改状态status
                                ], "CKD", 0, $out);

                                if ($out['status'] == 1) {
                                    if ($is_stock == 1) {
                                        //库存品，待办数据推给库管
                                        $uids = Db::name('user_role')
                                            ->where(['is_del' => 0, 'roleid' => config('app.wsm_cgder_role'), 'status' => 1])
                                            ->column('uid');
                                        ProcessOrder::AddProcess(['id' => $rm, 'nickname' => $ri], [
                                            "order_type" => 'CKD',
                                            "order_code" => $outCode,//出库单号
                                            "order_id" => $ou,
                                            "order_status" => $out['status'],
                                            "before_status" => 0,
                                            'holder_id' => $data['apply_id'],
                                            'handle_user_list' => implode(',', $uids),
                                        ]);
                                    } else {
//                                        //非库存品推给供应商负责人
//                                        $supp = Db::name('supplier')
//                                            ->field('person,personid')
//                                            ->where('code', $ct['supplierNo'])
//                                            ->findOrEmpty();
                                        $supp = $userCommon->handle('sInfo',['code'=>$ct['supplierNo']]);

                                        ProcessOrder::AddProcess(['id' => $rm, 'nickname' => $ri], [
                                            "order_type" => 'CKD',
                                            "order_code" => $outCode,//出库单号
                                            "order_id" => $ou,
                                            "order_status" => $out['status'],
                                            "before_status" => 0,
                                            'holder_id' => $data['apply_id'],
                                            'wait_id' => $supp['data']['personid'],
                                            'wait_name' => $supp['data']['person'],
                                        ]);
                                    }


                                }else ProcessOrder::AddProcess(['id' => $rm, 'nickname' => $ri], [
                                    "order_type" => 'CKD',
                                    "order_code" => $outCode,//出库单号
                                    "order_id" => $ou,
                                    "order_status" => $out['status'],
                                    "before_status" => 0,
                                    'holder_id'=>$data['apply_id']
                                ]);

//                                $standing_book_data['outCode'] = $outCode;
                            }
                        } else throw new Exception('发货地址添加失败');
                    }
                }

                //复用sale::create()方法 -- end

                //处理完成
//                $order_import_from_c_db
//                    ->where('id', $c_data['id'])
//                    ->where('is_del', OIFCModel::$is_del_normal)
//                    ->save([
//                        'status' => $order_import_from_c_db::$status_success,
//                        'updatetime' => date('Y-m-d H:i:s'),
//                        'remark' => '',
//                        'updateid' => 0,
//                        'updater' => 'system',
//                        'orderCode' => $orderCode
//                    ]);

                //维护台账记录
                Db::name('standing_book')->insert(array_merge($standing_book_data, ['addtime' => date('Y-m-d H:i:s'), 'updatetime' => date('Y-m-d H:i:s'), 'standBookNo' => makeNo("IO")]));


                //回调更新有赞订单数据
                $callback_rs = $this->callbackYz(['status' => 6, 'id' => $c_data['id'], 'orderCode' => $orderCode]);
                if ($callback_rs['code'] != 0) throw new \Exception($callback_rs['message']);

                //将采购单数据塞入到队列中
                if ($this->cgd_data) {
                    $push_data = json_encode([
                        'supplierNo' => $this->cgd_data['supplierNo'],
                        'type' => 1,//1销售订单(采销的采购单),2上线结果
                        'data' => [
                            'cgdNo' => $this->cgd_data['cgdNo'],
                            'spuCode' => $this->cgd_data['spuCode'],
                            'good_name' => $this->cgd_data['good_name'],
                            'good_num' => $this->cgd_data['good_num'],
                            'good_price' => $this->cgd_data['good_price'],
                            'total_fee' => $this->cgd_data['total_fee'],
                            'weight' => $this->cgd_data['weight'],
                            'addtime' => $this->cgd_data['addtime'],
                        ],
                    ], JSON_UNESCAPED_UNICODE);
//                    Cache::store("redis")->handler()->lPush(Config::get('app.abutment_queue'), $push_data);
                }

                Db::commit();
				Cache::store('redis')->set("YzTimeLimit",0);
                $output->writeln(date('Y-m-d H:i:s') . '|处理成功,orderCode:' . $orderCode);

            } catch (\Exception $exception) {
                Db::rollback();

                $message[] = $exception->getMessage();

                //回调更新有赞订单数据
                $callback_rs = $this->callbackYz(['status' => 8, 'id' => $c_data['id'], 'reason' => $exception->getMessage()]);
                if ($callback_rs['code'] != 0) $message[] = $callback_rs['message'];
				Cache::store('redis')->set("YzTimeLimit",0);
                $output->writeln(date('Y-m-d H:i:s') . '|处理失败，' . implode(',', $message));
            }

        }
        Cache::store('redis')->set("YzTimeLimit",0,60);
        //没有需要处理的数据

    }


    //创建采购单
    private function createCgd($data = [], $rm = '0', $ri = '', array &$standing_book_data = [])
    {
        $cgdCode = makeNo("CG");

        $wsm = Db::name("warehouse_info")
            ->where(["supplierNo" => $data["supplierNo"], "companyNo" => $data['companyNo'], "is_del" => 0])
            ->find();
        if ($wsm == false) {
            $wsm_code = makeNo("WSM");
            $inwsm = [
                "wsm_code" => $wsm_code,
                "name" => $data['supplier_name'],
                "wsm_type" => 2,
                "supplierNo" => $data['supplierNo'],
                "addr" => "",
                "addrs_code" => "",
                "contactor" => 0,
                "contactor_name" => 0,
                "mobile" => "",
                "position" => "",
                "companyNo" => $data['companyNo'],
                "status" => 1,
                "is_del" => 0,
                "addtime" => date("Y-m-d H:i:s"),
                "updatetime" => date("Y-m-d H:i:s")
            ];
            $in = Db::name("warehouse_info")->insert($inwsm);
            if ($in == false) return false;

        } else {
            $wsm_code = $wsm['wsm_code'];
        }

        //判断该供应商是否开通了供应商账号
        $from_tag=checkHasAccountBySupplierNos([$data['supplierNo']]);

        $cg = [
            "cgdNo" => $cgdCode,
            "bkcode" => "",
            "wsm_code" => $wsm_code,
            "cgder" => $data['creater'],
            "cgder_id" => $data['createrid'],
            "spuCode" => $data['spuCode'],
            "skuCode" => $data['skuCode'],
            "good_name" => $data['good_name'],
            "good_num" => $data['good_num'],
            "good_price" => $data['sale_price'],
            "total_fee" => round($data['sale_price'] * $data['good_num'], 2),
            "origin_total" => round($data['sale_price'] * $data['good_num'], 2),
            "pakge_fee" => $data['pakge_fee'],
            "cert_fee" => $data['cert_fee'],
            "open_fee" => $data['open_fee'],
            "teach_fee" => $data['cost_fee'],
            "mark_fee" => $data['mark_fee'],
            "demo_fee" => $data['demo_fee'],
            "nake_fee" => $data['nake_fee'],
            "delivery_fee" => $data['delivery_fee'],
            "weight"=>$data['weight'],
            "diff_weight" => "0",
            "diff_fee" => "0",
            "gold_price" => $data['gold_price'],
            "supplierNo" => $data['supplierNo'],
            "supplier_name" => $data['supplier_name'],
            "companyNo" => $data['companyNo'],
            "companyName" => $data['companyName'],
            "send_status" => 1,
            "send_num" => 0,
            "wsend_num" => $data['good_num'],
            "remark" => "",
            "status" => 0,//0初始化
            "lasttime" => date("Y-m-d H:i:s"),
            "is_del" => 0,
            "order_type" => $data['order_type'],
            "open_type"=>$data['open_type'],
            "order_source" => $data['order_source'],
            "good_type" => $data['good_type'],
            "addtime" => date("Y-m-d H:i:s"),
            "updatetime" => date("Y-m-d H:i:s"),
            'good_createrid' => $data['good_createrid'],
            'good_creater' => $data['good_creater'],//商品创建人
            'from_tag' => isset($from_tag[$data['supplierNo']]) ? 2 : 1,//来源标签：1采销(默认),2供应商端
            'supplier_origin_price'=>$data['supplier_origin_price'],//供应商采购成本
	        'cgd_supplier_code'=>$data['cgd_supplier_code'],//采购供应商编号
	        'cgd_supplier_name'=>$data['cgd_supplier_name'],//采购供应商编号
	        'cgd_apply_id'=>$data['cgd_apply_id'],//竞单人
	        'cgd_apply_name'=>$data['cgd_apply_name'],//竞单人
        ];
        $up = Db::name("purchease_order")->insert($cg, true);
        if ($up) {
            $taxInfo = OrderTax::SpuCat( $data['spuCode'],1,1);
           if (!empty($taxInfo)) {
              (new \app\admin\model\OrderTax)->saveAll(array_map(function ($item)use($cgdCode){
                   $temp=[];
                   $temp['code']=$cgdCode;
                   $temp['spuCode']=$item['spuCode'];
                   $temp['tax_code']=$item['cat_code'];
                   $temp['tax_name']=$item['cat_name'];
                   $temp['tax_short_name']=$item['short_name'];
                   $temp['inv_tax']=$item['tax'];
                   $temp['inv_good_name']=$item['inv_good_name'];
                   return $temp;
              },$taxInfo));
            }
            //修改状态，添加待办
            ActionLog::logAdd(['id' => $rm, 'nickname' => $ri], [
                "order_code" => $cg['cgdNo'],//销售单code
                "status" => 0,//这里的status是之前的值
                "action_remark" => '',//备注
                "action_type" => "create"//新建create，编辑edit，更改状态status
            ], "CGD", $cg['status'], $cg);

            //当采购单是节点0待与供应商确认，推给供应商负责人
            ProcessOrder::AddProcess(['id' => $rm, 'nickname' => $ri], [
                "order_type" => 'CGD',
                "order_code" => $cg['cgdNo'],//销售单code
                "order_id" => $up,
                "order_status" => $cg['status'],
                "before_status" => 0,
                'holder_id'=>$data['createrid'],
                'wait_id'=>$data['createrid'],
                'wait_name'=>$data['creater'],
            ]);

            $standing_book_data['cgdNo'] = $cgdCode;

            $rela = [
                "orderCode" => $data['orderCode'],
                "cgdNo" => $cgdCode,
                "spuCode" => $data['spuCode'],
                "good_num" => $data['good_num'],
                "wsend_num" => $data['good_num'],
                "send_num" => 0,
                "wait_num" => 0,
                "status" => 1,
                "source" => 2
            ];
            $re = Db::name("order_num")->save($rela);
            if ($re == false) return false;
            else {
                $stokc = Db::name("good_stock")
                    ->where(['spuCode' => $data['spuCode'], "wsm_code" => $wsm_code, "is_del" => 0])
                    ->find();
                if ($stokc == false) {
                    $stokc = [
                        "spuCode" => $data['spuCode'],
                        "wsm_code" => $wsm_code,
                        "wait_in_stock" => $data['good_num'],
                        "wait_out_stock" => 0,
                        "usable_stock" => 0,
                        "intra_stock" => 0,
                        "total_stock" => 0,
                        "status" => 1,
                        "addtime" => date("Y-m-d H:i:s"),
                        "updatetime" => date("Y-m-d H:i:s")
                    ];
                } else {
                    $stokc['wait_in_stock'] += $data['good_num'];
                    $stokc['updatetime'] = date("Y-m-d H:i:s");
                }
                $stoc = Db::name("good_stock")->save($stokc);
                if ($stoc == false) return false;

                $good_data[] = ['good_log_code' => $cgdCode, "stock_id" => isset($stoc['id']) ? $stoc['id'] : Db::name("good_stock")->getLastInsID(), "type" => 1, 'stock' => $data['good_num'], "stock_name" => "wait_in_stock"];
                GoodLog::LogAdd(['id' => 0, 'nickname' => 'system'], $good_data, "CGD");

                $this->cgd_data = $cg;

                return true;
            }
        } else return false;

    }


    //创建.....
//    private function RelaCgd(array $outinfo = [], array &$standing_book_data = [])
//    {
//        //备库单
//        $cgd = Db::name("order_bk")
//            ->where([
//                ["spuCode", "=", $outinfo['spuCode']],
//                ["is_del", "=", 0],
//                ["balance_num", ">=", $outinfo['good_num']],
//                ['companyNo', "=", $outinfo['companyNo']]
//            ])
//            ->lock(true)
//            ->find();
//
//        if ($cgd == false) return false;
//
//        $cgdinfo = Db::name("purchease_order")->where(['cgdNo' => $cgd['cgdNo'], "is_del" => 0])->find();
//        if ($cgdinfo == false) return false;
//
//        //判断该供应商是否开通了供应商账号
//        $from_tag=checkHasAccountBySupplierNos([$cgdinfo['supplierNo']]);
//
//        $QrdCgd = [
//            "cgdNo" => makeNo("CG"),
//            "bkcode" => $cgdinfo['bkcode'],
//            'wsm_code' => $cgdinfo['wsm_code'],
//            "cgder_id" => $cgdinfo['cgder_id'],
//            "cgder" => $cgdinfo['cgder'],
//            "spuCode" => $cgdinfo['spuCode'],
//            "good_name" => $cgdinfo['good_name'],
//            "good_num" => $outinfo['good_num'],
//            "good_price" => $cgdinfo['good_price'],
//            "total_fee" => round($cgdinfo['good_price'] * $outinfo['good_num'], 2),
//            "pakge_fee" => $cgdinfo['pakge_fee'],
//            "cert_fee" => $cgdinfo['cert_fee'],
//            "open_fee" => $cgdinfo['open_fee'],
//            "delivery_fee" => $cgdinfo['delivery_fee'],
//            "mark_fee" => $cgdinfo['mark_fee'],
//            "teach_fee" => $cgdinfo['teach_fee'],
//            "nake_fee" => $cgdinfo['nake_fee'],
//            "demo_fee" => $cgdinfo['demo_fee'],
//            "weight" => $cgdinfo['weight'],
//            "diff_weight" => $cgdinfo['diff_weight'],
//            "diff_fee" => $cgdinfo['diff_fee'],
//            "gold_price" => $cgdinfo['gold_price'],
//            "supplierNo" => $cgdinfo['supplierNo'],
//            "supplier_name" => $cgdinfo['supplier_name'],
//            "companyNo" => $cgdinfo['companyNo'],
//            "send_status" => 3,
//            "send_num" => $outinfo['good_num'],
//            "wsend_num" => 0,
//            "remark" => $cgdinfo['remark'],
//            "status" => 3,//入库完成
//            "lasttime" => $cgdinfo['lasttime'],
//            "is_del" => 0,
//            "order_type" => $outinfo['order_type'],
//            "order_source" => $outinfo['order_source'],
//            "good_type" => $cgdinfo['good_type'],
//            "addtime" => date("Y-m-d H:i:s"),
//            "updatetime" => date("Y-m-d H:i:s"),
//            'good_createrid' => $outinfo['good_createrid'],
//            'good_creater' => $outinfo['good_creater'],//商品创建人
//            'from_tag' => isset($from_tag[$cgdinfo['supplierNo']]) ? 2 : 1,//来源标签：1采销(默认),2供应商端
//        ];
//        $insetrCgd = Db::name("purchease_order")->insert($QrdCgd);
//        if ($insetrCgd == false) return false;
//
//        $standing_book_data = array_merge($standing_book_data, [
//            'cgdNo' => $QrdCgd['cgdNo'],
//            'bk_code' => $QrdCgd['bkcode'],
//        ]);
//
//        $merge_num = Db::name("purchease_order")
//            ->where(["bkcode" => $cgdinfo['bkcode'], "order_type" => $outinfo['order_type'], "is_del" => 0])
//            ->where('order_source', '<>', 0)//0备库单不计算在内
//            ->field("sum(send_num)-sum(th_num) as num")
//            ->find();
//        $cgd['balance_num'] = $cgd['total_num'] - $merge_num['num'];
//        $cgd['merge_num'] = $merge_num['num'];
//        $cgd['updatetime'] = date("Y-m-d H:i:s");
//        $up = Db::name("order_bk")->save($cgd);
//        if ($up == false) return false;
//
//        $data = [
//            "orderCode" => $outinfo['orderCode'],
//            "cgdNo" => $QrdCgd['cgdNo'],
//            "spuCode" => $outinfo['spuCode'],
//            "companyNo" => $outinfo['companyNo'],
//            "good_num" => $outinfo['good_num'],
//            "wsend_num" => $outinfo['good_num'],
//            "send_num" => 0,
//            "wait_num" => $outinfo['good_num'],
//            "status" => 1,
//            "source" => 1,
//        ];
//        $order = Db::name("order_num")->save($data);
//        if ($order == false) return false;
//
//        $this->cgd_data = $QrdCgd;
//
//        return true;
//    }


    //与有赞项目回调
    private function callbackYz(array $data = [])
    {
        $host = Config::get('app.yz_domain');
        $response = curl_request($host . 'api/callback_order_by_stock', $data);
        return json_decode($response, true);
    }


}
